/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  board.cpp - board database class implementation
 *
 *  Version: $Id: Board.cpp 3004 2006-08-07 13:25:09Z orenk $
 *
 */

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <errno.h>

#include "Board.h"

#ifndef XML_STATUS_OK
#define XML_STATUS_OK 1
#endif

#define numb_el(x) (sizeof(x)/sizeof(x[0]))
namespace std {}; using namespace std;

map<string, Board::info>       Board::_brd_info;
map<string, Board::more_info>  Board::_brd_more_info;
map<Board::Btype, string>      Board::_brd_types;
list<string>                   Board::_orig_order;

/*
 *                    Format of configuration word
 *                    ============================
 *
 * .___________________________________________________________________________.
 * !             !                                                             !
 * ! bit number  !                    Functionality                            !
 * !             !                                                             !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     0       !   P2P mode                                                  !
 * !             !                   1 -  P2P                                  !
 * !             !                   0 - not P2P                               !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     1       !   In P2P mode                                               !
 * !             !                   1 - secondary Gamla                       !
 * !             !                   0 - primary Gamla                         !
 * !             !   In non-P2P mode                                           !
 * !             !                   1 - force logical links up                !
 * !             !                   0 - do not force logical links up         !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     2       !   RESERVED                                                  !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     3       !   Configured as Static Fabric or by Subnet Manager :        !
 * !             !                   1 - Static assignment                     !
 * !             !                   0 - Subnet manager configuration          !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     4-7     !   Lid for static assignment ( if bit #3 is set )            !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     8       !   RESERVED                                                  !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     9       !   InfiniFour board. Only relevant when Sleek family board   !
 * !             !   (1) is selected in the "Type of Board" field              !
 * !             !                   0 - The board is not InfiniFour           !
 * !             !                   1 - The board is InfiniFour               !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     10-14   !   Type of Board                                             !
 * !             !                   0  - Mellanox EVB                         !
 * !             !                   1  - Sleek family board (may be:          !
 * !             !                        Sleek1x or Sleek4x or InfiniFour)    !
 * !             !                   4  - Buffalo                              !
 * !             !                   5  - Server blade                         !
 * !             !                   6  - Nitro/Gamla                          !
 * !             !                   7  - Nitro/Anafa                          !
 * !             !                   8-63 - RESERVED                           !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     15      !   In P2P mode only ,primary Gamla - IO support :            !
 * !             !                   0 - Support IO transactions               !
 * !             !                   1 - Memory Bridge (does not support IO )  !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     16      !   Configurate Gamla as switch only                          !
 * !             !                   0 - Normal configuration                  !
 * !             !                   1 - Switch only configuration             !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     17      !   RESERVED SHOULD BE always 1                               !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     18-19   ! Sleek4x board specification. Only relevant when Sleek       !
 * !             ! family board  (1) is selected in the "Type of Board" field  !
 * !             !                   0 - The board is Sleek1x                  !
 * !             !                   1 - Reserved                              !
 * !             !                   2 - Reserved                              !
 * !             !                   3 - The board is Sleek4x                  !
 * !             !                                                             !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !             !                                                             !
 * !     20-30   !   RESERVED                                                  !
 * !             !                                                             !
 * !_____________!_____________________________________________________________!
 * !             !                                                             !
 * !     31      ! If set - FW is loaded from host                             !
 * !_____________!_____________________________________________________________!
 *
 */
#define SETERR(args) do { errmsg args ; return; } while(0)

typedef void (Board::*start_el_hnd)(const char *name, map<string,string>& atts);
typedef void (Board::*end_el_hnd)(const char *name);
typedef pair<string,start_el_hnd>  ss_t;
typedef pair<string,end_el_hnd>    se_t;



// Start element handlers per element names
static const ss_t _start_h[] = { ss_t("BoardConfiguration", &Board::start_ignore),
                                 ss_t("board",              &Board::start_board),
                                 ss_t("InsertBuiltinList",  &Board::start_blist),
                                 ss_t("GPIO_REFL_ENABLE",   &Board::start_el),
                                 ss_t("GPIO_REFL_PORT0",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT1",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT2",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT3",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT4",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT5",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT6",    &Board::start_el),
                                 ss_t("GPIO_REFL_PORT7",    &Board::start_el),
                                 ss_t("MTU1k1vl",           &Board::start_el),
                                 ss_t("MTU1k8vl",           &Board::start_el),
                                 ss_t("ca_data",            &Board::start_el),
                                 ss_t("ca_ucode",           &Board::start_el),
                                 ss_t("eep_order",          &Board::start_el),
                                 ss_t("eeprom",             &Board::start_el),
                                 ss_t("id",                 &Board::start_el),
                                 ss_t("p2p_prim_data",      &Board::start_el),
                                 ss_t("p2p_prim_ucode",     &Board::start_el),
                                 ss_t("p2p_scnd_data",      &Board::start_el),
                                 ss_t("p2p_scnd_ucode",     &Board::start_el),
                                 ss_t("poff",               &Board::start_el),
                                 ss_t("pwidth",             &Board::start_el),
                                 ss_t("rxreordering",       &Board::start_el),
                                 ss_t("swizling",           &Board::start_el),
                                 ss_t("txreordering",       &Board::start_el)
};

// Start element handlers per element names
static const se_t _end_h[] = { se_t("BoardConfiguration", &Board::end_ignore),
                               se_t("board",              &Board::end_board),
                               se_t("InsertBuiltinList",  &Board::end_blist),
                               se_t("GPIO_REFL_ENABLE",   &Board::end_el),
                               se_t("GPIO_REFL_PORT0",    &Board::end_el),
                               se_t("GPIO_REFL_PORT1",    &Board::end_el),
                               se_t("GPIO_REFL_PORT2",    &Board::end_el),
                               se_t("GPIO_REFL_PORT3",    &Board::end_el),
                               se_t("GPIO_REFL_PORT4",    &Board::end_el),
                               se_t("GPIO_REFL_PORT5",    &Board::end_el),
                               se_t("GPIO_REFL_PORT6",    &Board::end_el),
                               se_t("GPIO_REFL_PORT7",    &Board::end_el),
                               se_t("MTU1k1vl",           &Board::end_el),
                               se_t("MTU1k8vl",           &Board::end_el),
                               se_t("ca_data",            &Board::end_el),
                               se_t("ca_ucode",           &Board::end_el),
                               se_t("eep_order",          &Board::end_el),
                               se_t("eeprom",             &Board::end_el),
                               se_t("id",                 &Board::end_el),
                               se_t("p2p_prim_data",      &Board::end_el),
                               se_t("p2p_prim_ucode",     &Board::end_el),
                               se_t("p2p_scnd_data",      &Board::end_el),
                               se_t("p2p_scnd_ucode",     &Board::end_el),
                               se_t("poff",               &Board::end_el),
                               se_t("pwidth",             &Board::end_el),
                               se_t("rxreordering",       &Board::end_el),
                               se_t("swizling",           &Board::end_el),
                               se_t("txreordering",       &Board::end_el)
};

// start/end handler maps static initialization
static map<string,start_el_hnd>  start_h; //(&_start_h[0], &_start_h[numb_el(_start_h)]);
static map<string,end_el_hnd>    end_h; //  (&_end_h[0],   &_end_h[numb_el(_end_h)]);

/*
 * Static boards configuration (will be in file)
 */

static const struct Archs {
    char *name;
    Board::ARCH arch;
} _archs[] = {
    { "21108", Board::GAMLA },
    { "43132", Board::ANAFA }
};

static const struct {
    char      *name;
    u_int16_t mask;
} _pci [] = {
    { "64KB",  0xffff },
    { "128KB", 0xfffe },
    { "256KB", 0xfffc },
    { "512KB", 0xfff8 },
    { "1MB",   0xfff0 },
    { "2MB",   0xffe0 },
    { "4MB",   0xffc0 },
    { "8MB",   0xff80 },
    { "16MB",  0xff00 },
    { "32MB",  0xfe00 },
    { "64MB",  0xfc00 },
    { "128MB", 0xf800 },
    { "256MB", 0xf000 },
    { "512MB", 0xe000 },
    { "1GB",   0xc000 },
    { "2GB",   0x8000 }
};

static const struct {
    char        *name;
    Board::info info;
} _conf [] = {
    {
        "MTEVB21108-C08-R1.41",
        {
            "Gamla EVB",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 0, 1, 0},
             {1, 1, 1, 1, 1, 0, 1, 0}
            },
            {0, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main.cin",
            "main_p2p_prim.hx",
            "main_p2p.cft main_p2p.cin",
            "main_p2p_scnd.hx",
            "main_p2p.cft main_p2p.cin",
            "mtu1k_vl0.cft mtu1k_vl0.cin",
            "mtu1k_vlcap8.cft mtu1k_vlcap8.cin"
        }
    },
    {
        "MTEVB21108-C04-1BN",
        {
            "Sleek1x",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 0, 1, 0}
            },
            {1, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main.cin",
            "main_p2p_prim.hx",
            "main_p2p.cft main_p2p.cin",
            "main_p2p_scnd.hx",
            "main_p2p.cft main_p2p.cin",
            "mtu1k_vl0.cft mtu1k_vl0.cin",
            "mtu1k_vlcap8.cft mtu1k_vlcap8.cin"
        }
    },
    {
        "MTEVB21108-C02-2BN4",
        {
            "Sleek4x",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {1, 1},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 1, 1, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 0, 1, 0}
            },
            {1, 3, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main.cin",
            "main_p2p_prim.hx",
            "main_p2p.cft main_p2p.cin",
            "main_p2p_scnd.hx",
            "main_p2p.cft main_p2p.cin",
            "mtu1k_vl0.cft mtu1k_vl0.cin",
            "mtu1k_vlcap8.cft mtu1k_vlcap8.cin"
        }
    },
    {
        "JNI-HCA-IBP-1x02",
        {
            "IBP-1x02",
            Board::GAMLA,
            {0x51, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{1, 1, 1, 1, 1, 0, 0, 1}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 0, 1, 0}
            },
            {8, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main_mod.cft main.cin",
            "",
            "",
            "",
            "",
            "mtu1k_vl0.cft mtu1k_vl0.cin",
            "mtu1k_vlcap8.cft mtu1k_vlcap8.cin"
        }
    },
    {
        "MTEVB21108-C04-1BL",
        {
            "Elictronics",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 1, 1, 1}
            },
            {1, 0, 1},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main.cin",
            "main_p2p_prim.hx",
            "main_p2p.cft main_p2p.cin",
            "main_p2p_scnd.hx",
            "main_p2p.cft main_p2p.cin",
            "mtu1k_vl0.cft mtu1k_vl0.cin",
            "mtu1k_vlcap8.cft mtu1k_vlcap8.cin"
        }
    },
/* This is an old Buffalo
    {
        "MTEVB43132-C08",
        {
            "Buffalo",
            Board::ANAFA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {0, 0, 0, 0, 0, 0, 0, 0},
             {0, 0, 0, 0, 0, 0, 0, 0}
            },
            {4, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_anafa.hx start2_anafa.hx main_anafa.hx",
            "main_anafa.cft main_anafa.cin",
            "",
            "",
            "",
            "",
            "",
            ""
        }
    },
*/
    {
        "MTEVB43132-C08",
        {
            "Buffalo",
            Board::ANAFA,
            {0x55, 0, 4, {0x55, 0x56, 0x57, 0x54}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {0, 0, 0, 0, 0, 0, 0, 0},
             {0, 0, 0, 0, 0, 0, 0, 0}
            },
            {9, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_anafa.hx start2_anafa.hx main_anafa.hx",
            "main_anafa.cft main_anafa.cin main_anafa.ccf",
            "",
            "",
            "",
            "",
            "",
            ""
        }
    },
    {
        "MTEVB21108-MSB",
        {
            "Server Blade (Nitro)",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 1, 1, 1}
            },
            {5, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_server_blade.hx start2_server_blade.hx main_server_blade.hx",
            "main.cft main.cin",
            "",
            "",
            "",
            "",
            "",
            ""
        }
    },
    {
        "MTEVB43132-MSX4/G",
        {
            "Nitrex/Gamla",
            Board::GAMLA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 1, 1, 1}
            },
            {6, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_ca.hx start2_ca.hx main_ca.hx",
            "main.cft main.cin",
            "",
            "",
            "",
            "",
            "",
            ""
        }
    },
    {
        "MTEVB43132-MSX4/A",
        {
            "Nitrex/Anafa",
            Board::ANAFA,
            {0x55, 0, 2, {0, 0, 0, 0}},
            {0, 0},
            {{0, 0, 0, 0, 0, 0, 0, 0}},
            {0, 0, 0, 0,
             {1, 1, 1, 1, 1, 1, 1, 1},
             {1, 1, 1, 1, 1, 1, 1, 1}
            },
            {7, 0, 0},
            {0, {
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
                {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
            }},
            "start1_anafa.hx start2_anafa.hx main_anafa.hx",
            "main_anafa.cft main_anafa.cin",
            "",
            "",
            "",
            "",
            "",
            ""
        }
    }
};


////////////////////////////////////////////////////////////////////////
Board::Board()
{
  // Init static db:
  // Windows does not define map init from vectors - need to insert one at a time.
  //static map<string,start_el_hnd>  start_h(&_start_h[0],  &_start_h[numb_el(_start_h)]);
  //static map<string,end_el_hnd>  end_h(&_end_h[0], &_end_h[numb_el(_end_h)]);

  if (start_h.empty()) {
    u_int32_t i;
    for(i = 0; i < numb_el(_start_h); i++) {
      start_h.insert(_start_h[i]);
    }

    for(i = 0; i < numb_el(_end_h); i++) {
      end_h.insert(_end_h[i]);
    }
  }
} // Board::Board


////////////////////////////////////////////////////////////////////////
Board::~Board()
{
} // Board::~Board


////////////////////////////////////////////////////////////////////////
void Board::fill_blist()
{
    more_info mi;
    fill_more_info_default(mi);
    for (unsigned int i=0; i<numb_el(_conf); i++)
    {
        _brd_info[_conf[i].name]              = _conf[i].info;
        _brd_more_info[_conf[i].name]         = mi;
        _brd_types[Btype(_conf[i].info.type)] = string(_conf[i].name);
        _orig_order.push_back(_conf[i].name);
    }
} // Board::fill_blist


////////////////////////////////////////////////////////////////////////
void Board::fill_more_info_default(Board::more_info& mi)
{
    mi.host_load = false;
    mi.force_link = false;
    mi.static_lid = false;
    mi.static_lid_1st = 0;
    mi.SCLDivider = 42;
    mi.StabTime = 200;
    mi.OutDivider = 1;
    mi.DividerNumer = 6;
    mi.DividerDenom = 0;
    mi.Bypass = false;
    mi.PowerDown = false;
    mi.OutEna = false;
    mi.EcOut = false;
    mi.r0FreqDiv = 1;
    mi.r0Bypass = false;
    mi.r0ErOut = false;
    mi.r1FreqDiv = 1;
    mi.r1Bypass = false;
    mi.r1ErOut = false;
} // Board::fill_more_info_default


////////////////////////////////////////////////////////////////////////
void Board::print_names()
{
    printf("\nThe following board names are known:\n");
    for (list<string>::iterator i=_orig_order.begin(); i!=_orig_order.end(); ++i)
    {
        printf("    %s (%s)\n", i->c_str(), _brd_info[*i].nick);
    }
    printf("\n");
} // Board::print_names


////////////////////////////////////////////////////////////////////////
bool Board::read(const char *fname)
{
    _fname = fname;

    ifstream fp(fname);
    if (!fp)
        return errmsg("Can't open file \"%s\": %s",
                      _fname.c_str(), strerror(errno));

    XML_Parser     parser = XML_ParserCreate(0);
    XML_SetUserData(parser, this);
    XML_SetElementHandler(parser, &Board::s_start_el, &Board::s_end_el);
    XML_SetCharacterDataHandler(parser, &Board::s_chars);

    h_start_doc();

    string rbuf;
    for (_line=1; getline(fp, rbuf); _line++)
    {
        rbuf.append("\n");
        if (XML_Parse(parser, rbuf.c_str(), rbuf.length(), 0) != XML_STATUS_OK)
            return errmsg("File: %s, Line: %d - %s", _fname.c_str(), _line,
                          XML_ErrorString(XML_GetErrorCode(parser)));
        if (err())
            return false;
    }

    if (XML_Parse(parser, 0, 0, 1) != XML_STATUS_OK)
        return errmsg("File: %s, Line: %d (End of file) - %s", _fname.c_str(),
                      _line, XML_ErrorString(XML_GetErrorCode(parser)));

    if (err())
        return false;

    h_end_doc();
    XML_ParserFree(parser);

    return true;
} // Board::read


////////////////////////////////////////////////////////////////////////
void Board::h_start_el(const XML_Char *name, const XML_Char **char_atts)
{
    _cur_txt = "";

    // Something failed before - do nothing here
    if (err())
        return;

    // Store original attributes in original order - some element needs that
    _char_atts = (XML_Char **)char_atts;

    // Prepare attributes map
    map<string,string> atts;
    for (int ai = 0; char_atts[ai];  ai+=2)
        atts[char_atts[ai]] = char_atts[ai+1];

    // Now call handler per element name
    map<string,start_el_hnd>::iterator hit = start_h.find(name);
    if (hit != start_h.end())
    {
        (this->*(hit->second))(name, atts);
        return;
    }

    // All other (unhandled) tags
    SETERR(("File: %s, Line: %d - tag \"%s\" is invalid.",
	    _fname.c_str(), _line, name));
} // Board::h_start_el


////////////////////////////////////////////////////////////////////////
void Board::h_end_el(const XML_Char *name)
{
    if (err())
    {
        // Something failed before - cleanup up
        return;
    }

    // Now call handler per element name
    map<string,end_el_hnd>::iterator hit = end_h.find(name);
    if (hit != end_h.end())
    {
        (this->*(hit->second))(name);
        return;
    }

    // All other (unhandled) tags
    SETERR(("File: %s, Line: %d - tag \"%s\" is invalid.",
	    _fname.c_str(), _line, name));
} // Board::h_end_el


////////////////////////////////////////////////////////////////////////
void Board::h_start_doc()
{
    err_clear();
    _in_board = false;
    _cur_txt = "";
} // Board::h_start_doc


////////////////////////////////////////////////////////////////////////
void Board::h_end_doc()
{
} // Board::h_end_doc


////////////////////////////////////////////////////////////////////////
void Board::h_chars(const XML_Char *s, int len)
{
    // Something failed before - do nothing here
    if (err())
        return;

    _cur_txt += string(s, len);
} // Board::h_chars


////////////////////////////////////////////////////////////////////////
void Board::start_blist(const char *, map<string,string>&)
{
    if (_in_board)
        SETERR(("File: %s, Line: %d - InsertBuiltinList can not be "
		"inside board definition", _fname.c_str(), _line));
} // Board::start_blist


////////////////////////////////////////////////////////////////////////
void Board::end_blist(const char *)
{
    fill_blist();
} // Board::end_blist


////////////////////////////////////////////////////////////////////////
void Board::start_el(const char *name, map<string,string>&)
{
    if (!_in_board)
        SETERR(("File: %s, Line: %d - Tag \"%s\" allowed only "
		"inside board definition.\n", _fname.c_str(), _line, name));
} //  Board::start_el


////////////////////////////////////////////////////////////////////////
void Board::end_el(const char *name)
{
    if (!strcmp(name, "ca_ucode"))
        _info.ca_ucode = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "ca_data"))
        _info.ca_data = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "p2p_prim_ucode"))
        _info.p2p_prim_ucode = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "p2p_prim_data"))
        _info.p2p_prim_data = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "p2p_scnd_ucode"))
        _info.p2p_scnd_ucode = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "p2p_scnd_data"))
        _info.p2p_scnd_data = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "MTU1k1vl"))
        _info.MTU1k1vl = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "MTU1k8vl"))
        _info.MTU1k8vl = strdup(_cur_txt.c_str());
    else if (!strcmp(name, "id"))
    {
        Board::btype *b = new Board::btype;
        b->type1 = atoi(_cur_txt.c_str());
        b->type2 = 0;
        b->type3 = 0;
        memcpy(&_info.type, b, sizeof(Board::btype));
    }
    else if (!strcmp(name, "eeprom"))
    {
        char *endp;
        list<string> vals = splitl(_cur_txt);
        if (vals.size() != 3)
            SETERR(("File: %s, Line: %d - Tag eeprom must contain exactly 3 fields",
		    _fname.c_str(), _line));
        list<string>::iterator it = vals.begin();
        char *p = (char *)it->c_str();
        _info.eeproms.i2c_addr = strtol(p, &endp, 0);
        if (*endp)
            SETERR(("File: %s, Line: %d - \"%s\" is invalid constant",
		    _fname.c_str(), _line, p));
        ++it;
        _info.eeproms.size_ind = atoi(it->c_str());
        ++it;
        _info.eeproms.amount = atoi(it->c_str());
    }
    else if (!strcmp(name, "eep_order"))
    {
        char *endp;
        int  ind=0;
        list<string> vals = splitl(_cur_txt);
        if (vals.size() != 4)
            SETERR(("File: %s, Line: %d - Tag eep_order must contain exactly 4 fields",
		    _fname.c_str(), _line));
        list<string>::iterator it = vals.begin();
        for (; it != vals.end(); ++it)
        {
            char *p = (char *)it->c_str();
            _info.eeproms.eep_order[ind++] = strtol(p, &endp, 0);
            if (*endp)
                SETERR(("File: %s, Line: %d - \"%s\" is invalid constant",
			_fname.c_str(), _line, p));
        }
    }
    else if (!strcmp(name, "GPIO_REFL_ENABLE"))
        _info.gpio_refl.enable = true;
    else if (!strncmp(name, "GPIO_REFL_PORT", strlen("GPIO_REFL_PORT")))
    {
      //Regexp port_exp("^GPIO_REFL_PORT([0-7])$");
      int len =  strlen("GPIO_REFL_PORT");
      //if (port_exp.match(name))
      if (name[len] >= '0' && name[len] <= '7')
        {
            char *endp;
            int ind = 0;
            int port = name[len] - '0'; // atoi(port_exp.cap(1).c_str());
            list<string> vals = splitl(_cur_txt);
            if (vals.size() != 4)
                SETERR(("File: %s, Line: %d - Tag GPIO_REFL_PORTx must contain exactly 4 fields",
			_fname.c_str(), _line));
            list<string>::iterator it = vals.begin();
            for (; it != vals.end(); ++it,++ind)
            {
                char *p = (char *)it->c_str();
                switch(ind)
                {
                case 0:
		  _info.gpio_refl.port[port].enable = (strtol(p, &endp, 0) != 0);
                    break;
                case 1:
		  _info.gpio_refl.port[port].polarity = (strtol(p, &endp, 0) != 0);
                    break;
                case 2:
		  _info.gpio_refl.port[port].ordering = (strtol(p, &endp, 0) != 0);
                    break;
                case 3:
                    _info.gpio_refl.port[port].pin = strtol(p, &endp, 0)&0x3f;
                    break;
                }
                if (*endp)
                    SETERR(("File: %s, Line: %d - \"%s\" is invalid constant",
			    _fname.c_str(), _line, p));
            }
        }
        else
            SETERR(("File: %s, Line: %d - Invalid GPIO_REFL_PORTx tag - \"%s\"",
		    _fname.c_str(), _line, name));
    }
    else
    {
        // Now we don't need spaces inside value
      vector<char>  _new_cur_txt(_cur_txt.size());
        char       *p = &_new_cur_txt[0];
        for (char *pp = (char *)_cur_txt.c_str(); *pp; pp++)
            if (!strchr("\r\n\t ", *pp))
                *p++ = *pp;
        *p = 0;

        if (!strcmp(name, "pwidth"))
        {
            Board::p_width *pw = new Board::p_width;
            pw->upper4x = _new_cur_txt[0] == '0' ? false : true;
            pw->lower4x = _new_cur_txt[1] == '0' ? false : true;
            memcpy(&_info.pwidth, pw, sizeof(Board::p_width));
        }
        else if (!strcmp(name, "poff"))
        {
            Board::p_off *poff = new Board::p_off;
            for (int i=0; i<8; i++)
                poff->off[i] = _new_cur_txt[i] == '0' ? false : true;
            memcpy(&_info.poff, poff, sizeof(Board::p_off));
        }
        else if (!strcmp(name, "swizling"))
        {
            _swiz_reord.swzT0 = _new_cur_txt[0] == '0' ? false : true;
            _swiz_reord.swzT1 = _new_cur_txt[1] == '0' ? false : true;
            _swiz_reord.swzR0 = _new_cur_txt[2] == '0' ? false : true;
            _swiz_reord.swzR1 = _new_cur_txt[3] == '0' ? false : true;
        }
        else if (!strcmp(name, "txreordering"))
        {
            for (int i=0; i<8; i++)
                _swiz_reord.reordT[i] = _new_cur_txt[i]=='0' ? false : true;
        }
        else if (!strcmp(name, "rxreordering"))
        {
            for (int i=0; i<8; i++)
                _swiz_reord.reordR[i] = _new_cur_txt[i]=='0' ? false : true;
        }
        else
            SETERR(("File: %s, Line: %d - Invalid tag \"%s\"",
		    _fname.c_str(), _line, name));
    }
} //  Board::end_el


////////////////////////////////////////////////////////////////////////
void Board::start_board(const char *, map<string,string>& atts)
{
    _board_name = atts["name"];
    string nick = atts["nick"];
    string arch = atts["arch"];
    if (_board_name.empty())
        SETERR(("File: %s, Line: %d - Board name must be specified",
		_fname.c_str(), _line));
    if (nick.empty())
        SETERR(("File: %s, Line: %d - Board nickname must be specified",
		_fname.c_str(), _line));
    if (arch.empty())
        SETERR(("File: %s, Line: %d - Board architecture must be specified",
		_fname.c_str(), _line));

    _in_board = true;
    _info.arch = UNKNOWN;

    // Find architecture
    for (unsigned int i=0; i<numb_el(_archs); i++)
    {
        if (!strcmp(_archs[i].name, arch.c_str()))
        {
            _info.arch = _archs[i].arch;
            break;
        }
    }
    if (_info.arch == UNKNOWN)
        SETERR(("File: %s, Line: %d - Architecture \"%s\" is unknown.",
		_fname.c_str(), _line, arch.c_str()));

    // Set default _info and _swiz_reord
    memset(&_info, 0, sizeof(_info));
    _info.nick             = strdup(nick.c_str());
    _info.eeproms.i2c_addr = 0x55;
    _info.eeproms.size_ind = 0;
    _info.eeproms.amount   = 2;
    _info.eeproms.eep_order[0]     = 0x55;
    _info.eeproms.eep_order[1]     = 0x56;
    _info.eeproms.eep_order[2]     = 0x57;
    _info.eeproms.eep_order[3]     = 0x54;
    _info.gpio_refl.enable = false;
    _info.ca_ucode         = "";
    _info.ca_data          = "";
    _info.p2p_prim_ucode   = "";
    _info.p2p_prim_data    = "";
    _info.p2p_scnd_ucode   = "";
    _info.p2p_scnd_data    = "";
    _info.MTU1k1vl         = "";
    _info.MTU1k8vl         = "";
    memset(&_swiz_reord, 0, sizeof(Board::swiz_reord));

    // More info record
    fill_more_info_default(_more_info);
} // Board::start_board


////////////////////////////////////////////////////////////////////////
void Board::end_board(const char *)
{
    _in_board = false;
    memcpy(&_info.swre, &_swiz_reord, sizeof(Board::swiz_reord));
    _brd_info[_board_name]        = _info;
    _brd_more_info[_board_name]   = _more_info;
    _brd_types[Btype(_info.type)] = _board_name;
    _orig_order.push_back(_board_name);
} // Board::end_board
