/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  board.h - board database class definitions
 *
 *  Version: $Id: Board.h 2909 2006-06-19 13:22:16Z orenk $
 *
 */

#ifndef BOARD_H
#define BOARD_H

#include <expat.h>
#include <list>
#include <map>

#include "utils.h"

class Board : public ErrMsg
{

public:
    // Types
    enum ARCH { UNKNOWN=0, GAMLA=21108, ANAFA=43132 };
    struct Eeproms {
        unsigned char i2c_addr;     // First EEPROM I2C slave address
        int           size_ind;     // size index; 0 for 32kbytes, 1 for 64kbytes
        int           amount;       // EEPROMs amount
        int           eep_order[4]; // EEPROM order (relevant if amount>2)
    };
    struct p_width {
        bool         upper4x;
        bool         lower4x;
    };
    struct p_off {
        bool         off[8];
    };
    struct swiz_reord {
        bool         swzT0;
        bool         swzT1;
        bool         swzR0;
        bool         swzR1;
        bool         reordT[8];
        bool         reordR[8];
    };
    struct btype {
        int          type1;
        int          type2;
        int          type3;
    };
    struct GPIO_REFL {
        bool         enable;
        struct GPIO_REFL_PORT {
            bool     enable;
            bool     polarity;
            bool     ordering;
            int      pin;
        }            port[8];
    };
    class Btype {
    public:
        Btype() : type1(0),type2(0),type3(0) {}
        Btype(const int t1, const int t2, const int t3) :
            type1(t1),type2(t2),type3(t3) {}
        Btype(const btype b) : type1(b.type1),type2(b.type2),type3(b.type3) {}
        int          type1;
        int          type2;
        int          type3;
        bool operator<(const Btype b) const
        {
            return (type1*10000 + type2*100 + type3) < 
                (b.type1*10000 + b.type2*100 + b.type3);
        }
    };
    struct info {
        char         *nick;
        ARCH         arch;
        Eeproms      eeproms;
        p_width      pwidth;
        p_off        poff;
        swiz_reord   swre;
        btype        type;
        GPIO_REFL    gpio_refl;
        char         *ca_ucode;
        char         *ca_data;
        char         *p2p_prim_ucode;
        char         *p2p_prim_data;
        char         *p2p_scnd_ucode;
        char         *p2p_scnd_data;
        char         *MTU1k1vl;
        char         *MTU1k8vl;
    };

    struct more_info {
        bool      host_load;    // Load FW from host
        bool      force_link;   // Force links up
        bool      static_lid;   // Static LID assigment
        u_int8_t  static_lid_1st;// First static LID
        // PLL
        u_int32_t SCLDivider;   // Serial clock divider
        u_int32_t StabTime;     // PLL stabilization time
        u_int8_t  OutDivider;   // Output divider
        u_int8_t  DividerNumer; // Divider numer.
        u_int8_t  DividerDenom; // Divider denomin.
        bool      Bypass;       // Bypass
        bool      PowerDown;    // Power down
        bool      OutEna;       // Output enable
        bool      EcOut;        // Enable output pin
        u_int8_t  r0FreqDiv;    // r0 - reference clock divider
        bool      r0Bypass;     // r0 - reference clock bypass
        bool      r0ErOut;      // r0 - ???
        u_int8_t  r1FreqDiv;    // r1 - reference clock divider
        bool      r1Bypass;     // r1 - reference clock bypass
        bool      r1ErOut;      // r1 - ???
    };

    // Methods
    Board();
    ~Board();

    // API
    bool  read(const char *fname);
    void  print_names();
    
    const char* get_first_board() { 
	return _orig_order.empty() ? "" : _orig_order.begin()->c_str();
    }

    info* get(const char *s)
    {
        std::map<std::string, info>::iterator it = _brd_info.find(s);
        return it == _brd_info.end() ? 0 : &it->second;
    }
    more_info* get_more(const char *s)
    {
        std::map<std::string, more_info>::iterator it = _brd_more_info.find(s);
        return it == _brd_more_info.end() ? 0 : &it->second;
    }

    // XML callbacks
    static void s_start_el(void *userData, const XML_Char *name,
                           const XML_Char **atts)
        { ((Board*)userData)->h_start_el(name, atts); }
    static void s_end_el(void *userData, const XML_Char *name)
        { ((Board*)userData)->h_end_el(name); }
    static void s_chars(void *userData, const XML_Char *s, int len)
        { ((Board*)userData)->h_chars(s, len); }

    // Real methods called on callbacks
    void h_start_el(const XML_Char *name, const XML_Char **atts);
    void h_end_el(const XML_Char *name);
    void h_start_doc();
    void h_end_doc();
    void h_chars(const XML_Char *s, int len);

    // Parameter-specific methods
    void start_ignore(const char *, std::map<std::string,std::string>& ) {}
    void end_ignore(const char *)                                        {}
    void start_board(const char *, std::map<std::string,std::string>&);
    void end_board(const char *);
    void start_el(const char *, std::map<std::string,std::string>&);
    void end_el(const char *);
    void start_blist(const char *, std::map<std::string,std::string>&);
    void end_blist(const char *);

private:
    void fill_blist();
    void fill_more_info_default(Board::more_info& _more_info);

    static std::map<std::string, info>       _brd_info;
    static std::map<std::string, more_info>  _brd_more_info;
    static std::map<Btype, std::string>      _brd_types;
    static std::list<std::string>            _orig_order;

    // Parser data
    std::string        _board_name;
    std::string        _fname;
    int                _line;
    bool               _in_board;
    XML_Char           **_char_atts;
    std::string        _cur_txt;
    Board::info        _info;
    Board::more_info   _more_info;
    Board::swiz_reord  _swiz_reord;
};

#endif
