/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  Eeprom.cpp - EEPROM class implementation
 *
 *  Version: $Id: Eeprom.cpp 3004 2006-08-07 13:25:09Z orenk $
 *
 */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#include <vector>
#include <list>
#include <string>

#include "Board.h"
#include "Eeprom.h"
#include "utils.h"

#define MAX_STR 1024
static char str[MAX_STR];
#define numb_el(x) (sizeof(x)/sizeof(x[0]))
namespace std {}; using namespace std;

char *Eeprom::versionID = "1.0.0";
char *Eeprom::cvsID  = "$Id: Eeprom.cpp 3004 2006-08-07 13:25:09Z orenk $";
char *Eeprom::dateID = "$Date: 2006-08-07 16:25:09 +0300 (Mon, 07 Aug 2006) $";

/*
 * Extended configuration section layout
 * -------------------------------------
 */
struct cnf {
    u_int32_t guid[2];
    u_int32_t word;
    u_int32_t RESERVED;
    char      bsn[48];
    u_int16_t pci_mask;
    u_int16_t port_disable;
    u_int32_t sw_re_mask;
    u_int32_t cff_ptr;        // Relevant only for Anafa/IB burning
    u_int32_t curr_fw_ver;    // Relevant only for Anafa/IB burning
    u_int32_t ver_prog_date;  // Relevant only for Anafa/IB burning
    u_int8_t  eep_order[4];   // EEPROMs order
    u_int8_t  psid[16];       // psid for fw management tool
    char      RESERVED2[152];
};

#if defined(__ia64__) || defined(__x86_64)
typedef u_int64_t ptr_t;
#else
typedef u_int32_t ptr_t;
#endif

////////////////////////////////////////////////////////////////////////
Eeprom::Eeprom(const Board::info *info, const Board::more_info *more_info,
               const char *fwdir) :
    _fwdir(fwdir), ucode_chunks(0), 
    _info((Board::info *)info),
    _more_info((Board::more_info *)more_info),
    _mem(fwdir), _mem1(fwdir), _ee_image(4*0x8000), _ee_use(4*0x8000)
{
    ee_size = (info->eeproms.size_ind+1)*0x8000;
    ee_amount = info->eeproms.amount;
    ee_first_addr = info->eeproms.i2c_addr;
} // Eeprom::Eeprom


////////////////////////////////////////////////////////////////////////
Eeprom::~Eeprom()
{
} // Eeprom::~Eeprom


////////////////////////////////////////////////////////////////////////
bool Eeprom::set_version(const char *ver)
{
    unsigned int i;

    if (strlen(ver) > VER_LEN-4)
        return errmsg("Version string is %d (maximal is %d)",
                      (int)strlen(ver), VER_LEN-4);
    for (i=0; i<VER_LEN; i++)
        Set(i+PLL_LEN, '\0');
    for (i=0; i<strlen(ver); i++)
        Set(i+PLL_LEN, ver[i]);
    version = ver;
    //Regexp vre("[0-9.]+");
    //if (!vre.match(version))
    //    return errmsg("Version string \"%s\" is invalid.", ver);
    //vli = splitl(vre.cap(0), ".");
    vli = splitl(ver, ". ");
    if (vli.size() > 3)
        return errmsg("Invalid version string %s. Should be MAJOR.MINOR.SUBMINOR",
		      //vre.cap(0).c_str());
		      ver);
    while(vli.size() < 3)
        vli.push_back("0");
    int ofs = PLL_LEN + VER_LEN - 1;
    int current = ee_size*ee_amount - CNF_LEN;
    current += (ptr_t)&((cnf*)0)->curr_fw_ver + 3;  // next after curr_fw_ver

    for (list<string>::iterator it = vli.begin(); it != vli.end(); ++it )
    {
        Set(ofs--, (u_int8_t)atoi(it->c_str()));
        Set(current--, (u_int8_t)atoi(it->c_str()));
    }
    Set(current, 0);  // Just to fill all word

    // Set date
    time_t     tt = time(0);
    struct tm *t = localtime(&tt);
    u_int32_t int_date = (t->tm_mday << 24) | ((t->tm_mon+1) << 16) | t->tm_year+1900;
    current = ee_size*ee_amount - CNF_LEN;
    current += (ptr_t)&((cnf*)0)->ver_prog_date;  // next after ver_prog_date
    Set4(current, int_date);

    return true;
}


/*
 * Read miscellaneous formats
 * --------------------------
 */
////////////////////////////////////////////////////////////////////////
bool Eeprom::read_hex(FILE *fp)
{
    char         str[MAX_STR], val[20], *endp, *p;
    unsigned int curr_addr, curr_value;
    int          line=0;

    //printf("Start parsing .hex file\n");

    // Format of string is FIXED
    //   VV_V_V_VVVV  // ADDR
    while(fgets(str, MAX_STR, fp))
    {
        line++;
        if (strncmp(&str[11], "  // ", 5))
            return errmsg("Wrong input format - line %d", line);

        // Opcode
        strncpy(val, str, 2);
        val[2] = '\0';
        strncat(val, &str[3], 1);
        strncat(val, &str[5], 1);
        strncat(val, &str[7], 4);
        curr_value = strtoul(val, &endp, 16);
        if (*endp != '\0')
            return errmsg("Line %d - wrong input format - hexa constant %s"
                          "has invalid characters", line, val);

        // Address
        str[20] = '\0';
        curr_addr = strtoul(&str[16], &endp, 16);
        if (*endp != '\0')
            return errmsg("Line %d - wrong input format - hexa constant %s"
                      "has invalid characters", line, &str[16]);

        if (((p=strchr(&str[21], '.')) != NULL) && !strncmp(p, "...", 3))
            continue;        // This is a "hole" area
        if (curr_addr == 0xffff)
        {
            str[21 + strcspn(&str[21], "\n\r")] = '\0';
            if (!set_version(&str[21]))
                return false;
        }
        if (curr_addr >= Ucode::ROM_START)
            continue;        // This is a ROM

        // The line is OK, get the opcode
        ucode[ucode_chunks].add(curr_addr, curr_value);
    }
    ucode_chunks++;
    return true;
} // Eeprom::read_hex


////////////////////////////////////////////////////////////////////////
#define FREAD(b, s, n, f) do                      \
{                                                 \
    if (fread((b), (s), (n), (f)) != ((size_t)n)) \
        return errmsg("Unexpected EOF.");         \
} while(0)
bool Eeprom::read_cfb(FILE *fp)
{
    static struct cfb_types
    {
        char *name;
        int  size;
    }
    cfb_types[] =
    {
        { "CR_space",            1  },
        { "FDB_UC_table",        1  },
        { "FDB_MC_table",        2  },
        { "FDB_CACHE_table",     64 },
        { "PCIT_CLT_node",       40 },
        { "PCIT_enable_table",   4  },
        { "PCIM_CLT_node",       44 },
        { "PCIM_enable_table",   4  },
        { "PCIM_response_table", 8  },
        { "GRH_table",           40 },
        { "ERP_DATA_table",      4  },
        { "ERP_CODE_table",      4  },
        { "DEVSTATE_CRFLD",      4  },
        { "DEVSTATE_PKTABLE",    2  },
        { "DEVSTATE_RSU",        4  }
    };
    const int PREAMBLE_SIZE=160;
    int       devid, offs, size;
    unsigned  id;

    //printf("Start parsing .cfb file\n");

    // Read preamble and ignore it
    FREAD(str, 1, PREAMBLE_SIZE, fp);
    FREAD(&devid, sizeof(int), 1, fp);
    //printf("Device id:%d\n", devid);
    while(1)
    {
        int rc = fread(&id, sizeof(int), 1, fp);
        if (!rc)
            return errmsg(strerror(errno));
        if (rc != 1)
            return errmsg("Unexpected EOF.");

        FREAD(&offs, sizeof(int), 1, fp);
        FREAD(&size, sizeof(int), 1, fp);
        //printf("id:%x, offs:%x, size:%x\n", id, offs, size);
        switch (id)
        {
        case 0:
        {
            // CR_space
            unsigned int *buf = new unsigned int[size/4];
            if (size%sizeof(int))
                return errmsg("Invalid record size (%d).", size);
            FREAD(buf, cfb_types[id].size, size, fp);
            for (int i=0; i<size/4; i++)
            {
                int ind = offs/4 + i;
                _mem.Set(ind, buf[i]);
            }
            delete [] buf;
            break;
        }
        default:
            if (id >= sizeof(cfb_types)/sizeof(cfb_types[0]))
                return errmsg("Invalid record type %d", id);
            errmsg("Record of type %d (%s) not supported yet.",
                   id, cfb_types[id].name);
        }
    }
} // Eeprom::read_cfb


////////////////////////////////////////////////////////////////////////
bool Eeprom::read_cin(FILE *fp)
{
    int        line=0;

    //printf("Start parsing .cin file\n");
    while(fgets(str, MAX_STR, fp))
    {
        char addr_str[10], value_str[10], *endp;
        unsigned int value, address;

        line++;

        // Skip comments or empty lines
        if (str[strspn(str, " \t\r")] == '#')
            continue;
        if (str[strspn(str, " \t\r")] == '\n')
            continue;

        if (sscanf(str, "%s%s", addr_str, value_str) != 2)
            return errmsg("Line %d - invalid format", line);
        address = strtoul(addr_str, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid value format", line);
        value = strtoul(value_str, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid value format", line);
        // mem1->Set(address, value);  // !!!
        _mem.Set(address, value);
    }
    return true;
} // Eeprom::read_cin


////////////////////////////////////////////////////////////////////////
bool Eeprom::read_cff(FILE *fp)
{
    int        line=0;

    //printf("Start parsing .cin file\n");
    while(fgets(str, MAX_STR, fp))
    {
        char addr_str[10], value_str[10], *endp;
        unsigned int value, address;

        line++;

        // Skip comments or empty lines
        if (str[strspn(str, " \t\r")] == '#')
            continue;
        if (str[strspn(str, " \t\r")] == '\n')
            continue;

        if (sscanf(str, "%s%s", addr_str, value_str) != 2)
            return errmsg("Line %d - invalid format", line);
        address = strtoul(addr_str, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid value format", line);
        value = strtoul(value_str, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid value format", line);
        _mem1.Set(address, value);
    }
    return true;
} // Eeprom::read_cff


////////////////////////////////////////////////////////////////////////
bool Eeprom::read_cft(FILE *fp)
{
    int        line=0;

    //printf("Start parsing .cft file\n");
    while(fgets(str, MAX_STR, fp))
    {
        char name[200], val[10], *endp;
        unsigned int value;

        line++;

        // Skip comments and empty lines
        if (str[strspn(str, " \t\r")] == '#')
            continue;
        if (str[strspn(str, " \t\r")] == '\n')
            continue;

        if (sscanf(str, "%s%s", name, val) != 2)
            return errmsg("Line %d - invalid format", line);
        value = strtoul(val, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid value format", line);
        _mem.Set(name, value);
    }
    return true;
} // Eeprom::read_cft


////////////////////////////////////////////////////////////////////////
bool Eeprom::read()
{
    typedef bool (Eeprom::*read_t)(FILE *fp);
    FILE          *fp;

    static struct read_table
    {
        char *suff;     // Suffix
        read_t read_f;  // read function
    } read_table[] =
    {
        { ".hex",  &Eeprom::read_hex },
        { ".hx",   &Eeprom::read_hex },
        { ".cff",  &Eeprom::read_cff },
        { ".ccf",  &Eeprom::read_cff },
        { ".cft",  &Eeprom::read_cft },
        { ".cfb",  &Eeprom::read_cfb },
        { ".cin",  &Eeprom::read_cin }
    };

    char *loaded_files[] = { _info->ca_ucode, _info->ca_data
                             /* , pinfo->MTU1k1vl, pinfo->MTU1k8vl */ };
    for (unsigned i=0; i<numb_el(loaded_files); i++)
    {
        vector<string> fl = splitv(loaded_files[i]);
        for (unsigned j=0; j<fl.size(); j++)
        {
	    string filename_str(_fwdir + "/" + fl[j]);
            const char *filename = filename_str.c_str();

            // Open file
            if ((fp = fopen(filename, "rb")) == NULL)
                return errmsg("Can't open file \"%s\" - %s",
                              filename, strerror(errno));
            //printf("Start reading file \"%s\"\n", filename);

            // Call read function according to file type (extension)
            unsigned k;
            for (k=0; k<numb_el(read_table); k++)
            {
                if (!strcmp(&filename[strlen(filename)-strlen(read_table[k].suff)],
                            read_table[k].suff))
                {
                    if (!(this->*read_table[k].read_f)(fp))
                        return false;
                    break;
                }
            }
            if (k >= numb_el(read_table))
                return errmsg("File \"%s\" - unknown file type", filename);

            //printf("Closing file \"%s\"\n", filename);
            fclose(fp);
        }
    }

    return true;
} // Eeprom::read


/*
 * Compile EEPROM image
 * --------------------
 */
////////////////////////////////////////////////////////////////////////
bool Eeprom::compile(const char* psid)
{
#if 0
    // Print all collected ucode
    for (int chunk=0; chunk<ucode_chunks; chunk++)
    {
        printf("\nUcode chunk #%d, %x - %x",
              chunk, ucode[chunk].first(), ucode[chunk].last());
        for (unsigned int i=ucode[chunk].first();
             i <= ucode[chunk].last(); i++)
            printf("CODE - ADDR:%04x <- %08x\n", i, ucode[chunk][i]);
    }
#endif

#if 0
    // Print all collected CR/Mem space
    printf("\n\nCR/Mem space:");
    for (unsigned int i=0; i<_mem.Size(); i++)
        if (_mem.used(i))
            printf("DATA - ADDR:%04x (%04x) <- %08x\n",
                  i, i*4, _mem[i]);

#endif
    /*
     * Determine where conf. section should be
     * and fill pointer to it
     */
    Set(PLL_LEN+VER_LEN, 0);
    u_int16_t last_i2c_slave = ee_first_addr + ee_amount - 1;
    correct_i2c_addr(last_i2c_slave);
    Set(PLL_LEN+VER_LEN+1, last_i2c_slave);
    Set2(PLL_LEN+VER_LEN+2, ee_size - CNF_LEN);

    // First tree mandatory sections
    _sections.push_back(Section("EEPROM_PLL", 0, PLL_LEN));
    _sections.push_back(Section("EEPROM_SWR", PLL_LEN, VER_LEN));
    _sections.push_back(Section("EEPROM_PTR", PLL_LEN+VER_LEN, PTR_LEN));

    // Set PLL
    u_int8_t          pll[PLL_LEN];
    memset(pll, 0, PLL_LEN);
    pll[0] = (u_int8_t)_more_info->SCLDivider;
    pll[10] = (u_int8_t)_more_info->StabTime;
    u_int32_t cConfig = 0;
    cConfig = _more_info->OutDivider;
    cConfig |= _more_info->DividerDenom << 2;
    cConfig |= _more_info->DividerNumer << 7;
    cConfig |= (_more_info->Bypass?1:0) << 16;
    cConfig |= (_more_info->PowerDown?1:0) << 17;
    cConfig |= (_more_info->OutEna?1:0) <<18;
    cConfig |= (_more_info->EcOut?1:0) << 23;
    pll[1] = cConfig & 0xff;
    pll[2] = (cConfig>>8) & 0xff;
    pll[3] = (cConfig>>16) & 0xff;
    pll[4] = _more_info->r0FreqDiv;
    pll[4] |= (_more_info->r0Bypass?1:0) << 7;
    pll[6] = (_more_info->r0ErOut?1:0) << 7;
    pll[7] = _more_info->r1FreqDiv;
    pll[7] |= (_more_info->r1Bypass?1:0) << 7;
    pll[9] = (_more_info->r1ErOut?1:0) << 7;
    for (unsigned int i=0; i<PLL_LEN; i++)
        Set(i, pll[i]);

    // Set GPIO refl. info
    if (_info->gpio_refl.enable)
    {
        _mem.Set("SWSMASYS.REGRAM.GDSR.RE", 1);
        for (int i=0; i<8; i++)
        {
            Str        s;
            const char *pref = "SWSMASYS.REGRAM.GDSR.PORT_STATE_CONF";

            _mem.Set(s.print("%s[%i].E", pref, i), _info->gpio_refl.port[i].enable);
            _mem.Set(s.print("%s[%i].P", pref, i), _info->gpio_refl.port[i].polarity);
            _mem.Set(s.print("%s[%i].O", pref, i), _info->gpio_refl.port[i].ordering);
            _mem.Set(s.print("%s[%i].GPIO", pref, i), _info->gpio_refl.port[i].pin);
        }
    }
    else
    {
        if (_mem.exists("SWSMASYS.REGRAM.GDSR.RE"))
        {
            _mem.Set("SWSMASYS.REGRAM.GDSR.RE", 0);
            for (int i=0; i<8; i++)
            {
                Str        s;
                const char *pref = "SWSMASYS.REGRAM.GDSR.PORT_STATE_CONF";

                _mem.Set(s.print("%s[%d].E", pref, i), 0);
                _mem.Set(s.print("%s[%d].P", pref, i), 0);
                _mem.Set(s.print("%s[%d].O", pref, i), 0);
                _mem.Set(s.print("%s[%d].GPIO", pref, i), 0);
            }
        }
    }

    // Prepare conf sections - cft and cin fiels
    gather_conf(_mem, conf_single, conf_mult);

    // Prepare conf sections - cff files
    gather_conf(_mem1, conf_single1, conf_mult1);

    // Ucode and static data
    int current=PLL_LEN+VER_LEN+PTR_LEN;
    int start2_start, start2_end;

    fill_ucode(current, 0);    // Start1
    start2_start = current;
    fill_ucode(current, 1);    // Start2
    fill_data(_mem, conf_single, conf_mult, current);  // Conf
    start2_end = current;
    jump_to_next_eeprom(current);

    //fwb_start = current;
    fill_ucode(current, 2);    // Main
    jump_to_next_eeprom(current);

    // store eeprom numbers to FW
    int eep_order_ptr = ee_size*ee_amount - CNF_LEN
        + (ptr_t)&((cnf*)0)->eep_order;
    _sections.push_back(Section("EEPROM_NUMBERS", eep_order_ptr, 4));
    for (int e_ind=0; e_ind<4; e_ind++)
      Set(eep_order_ptr++, (u_int8_t)_info->eeproms.eep_order[e_ind]);

    // store ver_prog_date
    int ver_prog_date_ptr = ee_size*ee_amount - CNF_LEN
        +  (ptr_t)&((cnf*)0)->ver_prog_date;
    _sections.push_back(Section("VER_PROG_DATE", ver_prog_date_ptr, 4));


    // handle psid
    int psid_ptr = ee_size*ee_amount - CNF_LEN
        +  (ptr_t)&((cnf*)0)->psid;

    _sections.push_back(Section("PSID", psid_ptr, PSID_LEN));
    int p_ind;
    int psid_len = psid ? strlen(psid) : 0 ;

    for (p_ind=0; p_ind < psid_len && p_ind < PSID_LEN ; p_ind++, psid_ptr++)
        Set(psid_ptr , (u_int8_t)psid[p_ind]);

    for (; p_ind < PSID_LEN ; p_ind++, psid_ptr++)
        Set(psid_ptr , 0);

    // Tree ucode chunks (CA or new P2P)
    if (ee_amount == 4)
    {
        // store CFF pointer
        int cff_ptr = ee_size*ee_amount - CNF_LEN
            + (ptr_t)&((cnf*)0)->cff_ptr;
        int cff_orig = cff_ptr;
        Set(cff_ptr++, 0);
        Set(cff_ptr++, current/ee_size + ee_first_addr);
        Set2(cff_ptr, current % ee_size);
        cff_ptr += 2;
        _sections.push_back(Section("CFF_PTR", cff_orig, cff_ptr-cff_orig));

        // store curr_fw_ver
        int curr_fw_ver_ptr = ee_size*ee_amount - CNF_LEN
            +  (ptr_t)&((cnf*)0)->curr_fw_ver;
        _sections.push_back(Section("CURR_FW_VER", curr_fw_ver_ptr, 4));

        // Fill CFF
        fill_data(_mem1, conf_single1, conf_mult1, current);

        // Burn start2 mirror to fourth eeprom
        jump_to_next_eeprom(current);

        int sect_start = current;
        for (int i = start2_start; i < start2_end; i++)
            Set(current++, _ee_image[i]);
        _sections.push_back(Section("Start2Mirror", sect_start, current-sect_start));
        //fwb_end = current;
    }

    // Set BSN/GUID sections (empty)
    current = ee_size*ee_amount - CNF_LEN + GUID_LEN + FLAGS_LEN;
    _bsn_offs = current % ee_size;
    _bsn_i2c = current/ee_size + ee_first_addr;
    for (unsigned int i=0; i<BSN_LEN; i++)
        Set(i+current, 0);
    _sections.push_back(Section("EEPROM_CONFIG_BSN", current, BSN_LEN));

    current = ee_size*ee_amount - CNF_LEN;
    _guid_offs = current % ee_size;
    _guid_i2c = current/ee_size + ee_first_addr;
    Set4(current, 0);
    Set4(current+4, 0);
    _sections.push_back(Section("EEPROM_CONFIG_GUID", current, 8));

    // Set configuration WORD
    u_int32_t         cnf[2];       // Conf. info (GPIO SHADOW)
    cnf[0] = CNF_INIT;
    cnf[1] = 0;
    cnf[0] |= ((_info->type.type1 & 0x1f) << 10) |
        ((_info->type.type2 & 0x03) << 18) |
        ((_info->type.type3 & 0x01) << 9);
    cnf[0] |= _more_info->host_load ? (int)HOST_LOAD : 0;
    cnf[0] |= _more_info->force_link ? (int)FORCE_LINK : 0;
    if (_more_info->static_lid)
    {
        cnf[0] |= STATIC_LID_FLAG;
        cnf[0] |= _more_info->static_lid_1st << STATIC_LID_SHIFT;
    }
    current = ee_size*ee_amount - CNF_LEN + GUID_LEN;
    Set4(current, cnf[0]);
    Set4(current+4, cnf[1]);
    _sections.push_back(Section("EEPROM_CONFIG_GPFLAGS", current, 8));

    return true;
} // Eeprom::compile


////////////////////////////////////////////////////////////////////////
void Eeprom::correct_i2c_addr(u_int16_t& i2c_slave)
{
    if (_info->eeproms.amount > 2)
    {
        u_int16_t ind = i2c_slave - ee_first_addr;
        i2c_slave = (u_int16_t)_info->eeproms.eep_order[ind];
    }
} //  Eeprom::correct_i2c_addr


////////////////////////////////////////////////////////////////////////
void Eeprom::jump_to_next_eeprom(int& current)
{
    if (ee_amount>1)
        current += (ee_size % current) ? (ee_size - (current % ee_size)) : 0;
} // Eeprom::jump_to_next_eeprom


////////////////////////////////////////////////////////////////////////
void Eeprom::gather_conf(Mem& m, list<Conf_element>& singl,
                         list<Conf_element>& mult)
{
    enum state {ST_IN, ST_OUT} state = ST_OUT;
    int                  from = 0;

    for (unsigned int i=0; i<m.Size(); i++)
    {
        switch (state)
        {
        case ST_IN:
            if (!m.used(i))
            {
                if (i - from == 1)
                    singl.push_back(Conf_element(from, from));
                else
                    add_mult_data(mult, from, i-1);
                state = ST_OUT;
            }
            break;
        case ST_OUT:
            if (m.used(i))
            {
                state = ST_IN;
                from = i;
            }
            break;
        }

    }
} // Eeprom::gather_conf


////////////////////////////////////////////////////////////////////////
void Eeprom::add_mult_data(list<Conf_element>& mult, int from, int to)
{
    if (to > from + MAX_MULT_DATA - 1)
    {
        add_mult_data(mult, from, from + MAX_MULT_DATA - 1);
        add_mult_data(mult, from + MAX_MULT_DATA, to);
    }
    else
        mult.push_back(Conf_element(from, to));
} // Eeprom::add_mult_data


////////////////////////////////////////////////////////////////////////
void Eeprom::fill_data(Mem& m, list<Conf_element>& singl,
                       list<Conf_element>& mult, int& current)
{
#if 0
    printf("\nSINGLE:\n");
    for (Conf_element *ce=singl.first(); ce; ce=singl.next())
    {
        printf("%x (%x)\n", ce->from, ce->from*4);
    }

    printf("\nMULTIPLY:\n");
    for (Conf_element *ce=mult.first(); ce; ce=mult.next())
    {
        printf("%x (%x) - %x (%x) : %x (%x)\n",
               ce->from, ce->from*4, ce->to, ce->to*4,
               ce->to-ce->from+1, (ce->to-ce->from+1)*4);
    }
#endif
    int conf_beg = current;
    current += 4;

    // Single configuration
    while (singl.size() > 0)
    {
        unsigned int cnt = 0;

        Set(current++, 0);                            // Object ID
        Set(current++, (singl.size() > MAX_CONF0) ? (int)MAX_CONF0 :
            singl.size());                     // Object length
        while(singl.size() &&  (++cnt <= MAX_CONF0))
        {
            Conf_element& ce = singl.front();
            Set2(current, (u_int16_t)ce.from);         // Address
            current += 2;
            Set4(current, m[ce.from]);      // Data
            current += 4;
            //printf("%02x: %x <- %x\n", cnt, ce.from, m[ce.from]);
            singl.pop_front();

        }
    }

    // Multiple configuration
    while (mult.size() > 0)
    {
        unsigned int cnt = 0;

        Set(current++, 12);                           // Object ID
        Set(current++, (mult.size() > MAX_CONF0) ? (int)MAX_CONF0 :
            mult.size());                      // Object length
        while(mult.size()  &&  (++cnt <= MAX_CONF0))
        {
            Conf_element& ce = mult.front();
            Set2(current, (u_int16_t)ce.from);    // Address
            current += 2;
            Set(current++, ce.to - ce.from + 1);  // Number of words
            //printf("%02x: %x, %x - ", cnt, ce.from, ce.to-ce.from+1);
            for (int i = ce.from; i <= ce.to; i++)
            {
                Set4(current, m[i]);           // Data
                current += 4;
                //printf("%x ", (*m)[i]);
            }
            //printf("\n");
            mult.pop_front();

        }
    }

    // write length and store section
    Set4(conf_beg, current-conf_beg);
    _sections.push_back(Section("EEPROM_CONF", conf_beg, current-conf_beg));
} // Eeprom::fill_data


////////////////////////////////////////////////////////////////////////
void Eeprom::fill_ucode(int& current, int chunk)
{
    int       initial_current = current;
    u_int32_t first = ucode[chunk].first();
    u_int32_t last  = ucode[chunk].last();
    u_int16_t extra = 0;
    u_int16_t start = (u_int16_t)first;
    char      name[20];
    Crc16     crc;

    // For any non-zero chunk remove
    // the common portion of ucode
    if (chunk > 0)
    {
        start = ucode[chunk][0] & 0xffff;  // Target of first "call" command
        first = start;                     // Skip this code
    }

    // Append to quads
    u_int16_t len   = last - first + 1;
    u_int16_t quads = len / 4;
    if (len%4)
    {
        extra = 4 - len%4;
        quads++;
    }

    // Header
    Set2(current, quads);
    crc.add(quads);
    current += 2;
    Set2(current, start);
    crc.add(start);
    current += 2;

    // Code itself
    for (unsigned int i32=first; i32 <= last; i32++)
    {
        Set4(current, ucode[chunk][i32]);
        crc.add(ucode[chunk][i32]);
        current += 4;
    }

    // Padding
    for (u_int16_t i16=0; i16<extra; i16++)
    {
        Set4(current, Ucode::pad_value);
        crc.add((u_int32_t)Ucode::pad_value);
        current += 4;
    }

    // CRC16
    Set2(current, crc.get());
    current += 2;

    // Register section
    sprintf(name, "EEPROM_UCODE%d", chunk+1);
    _sections.push_back(Section(name, initial_current, current-initial_current));
} // Eeprom::fill_ucode


////////////////////////////////////////////////////////////////////////
bool Eeprom::write(const char *brd, const char *fname)
{
    // Open image
    FILE *fp = fopen(fname, "w");
    if (!fp)
        return errmsg("Can't open \"%s\" - %s", fname, strerror(errno));

    // File header
    list<string>::iterator it = vli.begin();
    time_t                 tt = time(0);
    struct tm              *t = localtime(&tt);
    correct_i2c_addr(_guid_i2c);
    correct_i2c_addr(_bsn_i2c);
    fprintf(fp, "# Created by InfiniScale image compiler %s\n", versionID);
    //fprintf(fp, "#     cvs %s\n", cvsID);
    //fprintf(fp, "#     cvs %s\n", dateID);
    fprintf(fp, "# At %02d:%02d:%02d %02d/%02d/%04d\n#\n",
            t->tm_hour, t->tm_min, t->tm_sec, t->tm_mon+1, t->tm_mday,
            t->tm_year+1900);
    fprintf(fp, "# Board = %s (%s)\n#\n", brd, _info->nick);
    fprintf(fp, "\n\nDEVICE_TYPE MT43132\n");
    fprintf(fp, "MAJ_FW_VER %s\n", (*it++).c_str());
    fprintf(fp, "MIN_FW_VER %s\n", (*it++).c_str());
    fprintf(fp, "SUB_MIN_FW_VER %s\n", (*it++).c_str());

    // Device mapping:
    fprintf(fp, "\n");
    for (int i = 0; i < ee_amount ; i++) {
	fprintf(fp, "DEVMAP %08x %02x\n", 
		ee_size,
		_info->eeproms.eep_order[i]);
    }
    fprintf(fp, "\n");

    fprintf(fp, "REPLACE GUID %08x %02x\n", _guid_offs, _guid_i2c);
    fprintf(fp, "REPLACE BSN %08x %02x\n\n", _bsn_offs, _bsn_i2c);

    // Dump sections
    for (list<Section>::iterator it=_sections.begin(); it!=_sections.end(); ++it)
        if (!dump_section(fp, it->name.c_str(), it->start, it->len))
            return false;

    // Its all
    fclose(fp);
    return true;
} // Eeprom::write


////////////////////////////////////////////////////////////////////////
bool Eeprom::dump_section(FILE *fp, const char *name, int beg, int len)
{
    u_int16_t ee_addr = (u_int16_t)ee_first_addr;

    //printf("Dump section \"%s\"  0x%x-0x%x (length:0x%x)\n",
    //       name, beg, beg+len-1, len);

    // Calculate offset and EEPROM address
    int offset = beg;
    while (offset - ee_size >= 0)
    {
        offset -= ee_size;
        ee_addr++;
    }

    correct_i2c_addr(ee_addr);

    if (ee_addr > ee_first_addr+ee_amount-1)
     //orenk   return errmsg("Invalid offset 0x%x - too far.", beg);
	fprintf(stdout, "-W- Invalid offset 0x%x - may be too far (assuming consecutive eeproms i2c addresses).\n", beg);

    // Write section
    fprintf(fp, "START %s %08x %02x\n", name, offset, ee_addr);
    int cnt = 1;
    for (int i=beg; i<beg+len; i++)
    {
        fprintf(fp, " %02x", _ee_image[i]);
        if (!((cnt++) % BYTES_IN_STR))
            fprintf(fp, "\n");
    }
    if ((--cnt) % BYTES_IN_STR)
        fprintf(fp, "\n");
    fprintf(fp, "END %s\n\n", name);
    return true;
} // Eeprom::dump_section
