/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  Eeprom.h - EEPROM manipulation class definitions
 *
 *  Version: $Id: Eeprom.h 3390 2007-05-15 15:36:35Z orenk $
 *
 */

#ifndef EEPROM_H
#define EEPROM_H

#include <stdio.h>
//#include <endian.h>
#include <sys/types.h>

#include <list>
#include <string>

#include "Board.h"
#include "Mem.h"
#include "utils.h"

/*
 * Change endianess
 * ----------------
 */
static void SWAP32 (u_int32_t& X)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    X = ((X & 0x000000FFUL) << 24)  |
        ((X & 0x0000FF00UL) << 8)   |
        ((X & 0x00FF0000UL) >> 8)   |
        ((X & 0xFF000000UL) >> 24);
#else
X=X;
#endif
}
static void SWAP16 (u_int16_t& X)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    X = ((X & 0xFF00U) >> 8) |
        ((X & 0x00FFU) << 8);
#else
X=X;
#endif
}

////////////////////////////////////////////////////////////////////////
class Crc16
{
private:
    enum CONSTANTS {
        INIT = 0xffffffff           // Initialization value of CRC
    };
    u_int32_t reg;
    static u_int8_t reverse(u_int8_t c)
    {
        u_int8_t rc=0;
        for(int i=0; i<8; i++)
        {
            rc <<= 1;
            rc |=  c & 0x01;
            c >>= 1;
        }
        return rc;
    }
public:
    Crc16()              { reset();      }
    void clear()         { reset();      }
    void reset()         { reg = INIT;   }
    operator u_int16_t() { return get(); }
    void add(u_int8_t b)
    {
        for(int i=0; i<8; i++)
        {
            reg <<= 1;
            if ((b & 0x01) ^ ((reg & 0x00010000) == 0x00010000))
                reg = reg ^ 0x100b;
            b >>= 1;

        }
    }
    void add(u_int16_t value)
    {
        SWAP16(value);
        unsigned char *p = (unsigned char *)&value;
        for(unsigned int i=0; i<sizeof(u_int16_t); i++)
            add(*p++);
    }
    void add(u_int32_t value)
    {
        SWAP32(value);
        unsigned char *p = (unsigned char *)&value;
        for(unsigned int i=0; i<sizeof(u_int32_t); i++)
            add(*p++);
    }
    u_int16_t get()
    {
        u_int16_t rc = ~reg;
        rc = (reverse(rc >> 8) << 8) | reverse(rc);
        return rc;
    }
};


////////////////////////////////////////////////////////////////////////
class Ucode : public ErrMsg
{
private:
    unsigned int              max_addr;
    unsigned int              *code;  // Opcodes array
    unsigned int              f;      // First actual address
    unsigned int              l;      // Last actual address
public:

    enum CONSTANTS {
        fill_value = 0xFF000000,    // Dummy opcode for "..." lines
        pad_value  = 0,             // Padding opcode in holes
        ROM_START  = 0x8000         // ROM start address
    };

    Ucode(unsigned int max_addr_ = 0x4000)
    {
        max_addr = max_addr_;
        f = max_addr-1;
        l = 0;
        code = new unsigned int[max_addr];
        for (unsigned int i=0; i<max_addr; i++)
            code[i] = fill_value;
    }
    ~Ucode()                   { delete [] code; }

    bool add(unsigned int addr, unsigned int opcode)
    {
        if (addr >= max_addr)
            return errmsg("Invalid ucode address (0x%x)", addr);
        if (addr < f)
            f = addr;
        if (addr > l)
            l = addr;
        code[addr] = opcode;
        return true;
    }
    unsigned int first() { return f; }
    unsigned int last()  { return l; }
    unsigned int operator[] (unsigned int ind)
    {
        //if (ind < f)
        //    out->print(Out::ERR, "Index is too low (%d)", ind);
        //if (ind > l)
        //    out->print(Out::ERR, "Index is too high (%d)", ind);
        if (ind < f  ||  ind > l)
            abort();
        return code[ind];
    }
};


////////////////////////////////////////////////////////////////////////
class Eeprom : public ErrMsg
{
public:
    enum CONF { CNF_INIT=0x20000,     SWITCH_CNF=0x10000, FORCE_LINK=0x02,
                STATIC_LID_FLAG=0x08, STATIC_LID_SHIFT=4, HOST_LOAD=0x80000000 };
    enum CONSTANTS {
        max_ucode_chunks = 8,  // Maximal Ucode chunks amount
        BYTES_IN_STR = 16,     // For EEPROM image dump
        PLL_LEN = 16,          // Lenght of PLL section (in bytes)
        VER_LEN = 12,          // Max. lenght of version string
        PTR_LEN = 4,           // Length of "pointer to CONF" sect.
        CNF_LEN = 256,         // Lenght of CONF. section (bytes)
        GUID_LEN = 8,          // Lenght of GUID subsection
	PSID_LEN = 16,         // Lenght of PSID subsection
        FLAGS_LEN= 8,          // Length of cnf. flags subsection
        BSN_LEN=48,            // Length of BSN section
        MAX_MULT_DATA = 256,   // Max.length of mult.data configuration section
        MAX_CONF0 = 255,       // Max. number of configuration records in section
        COMPANY_ID = 0x0002c9  // Mellanox GUID base
    };

    Eeprom(const Board::info *i, const Board::more_info *m, const char *fwdir);
    ~Eeprom();

    bool read();
    bool compile(const char* psid);
    bool write(const char *brd, const char *fname);

    const char* fw_version() { return version.c_str();}

private:
    static char *versionID;           // Version of EEPROM class
    static char *cvsID;               // Internal version (cvs Id)
    static char *dateID;              // Internal date (cvs Date)

    // Section and sections list that misc. methods prepare
    // to final burn/write_to_file
    class Section {
    public:
        Section(const char *name_, int start_, int len_)
        {
            name = name_;
            start = start_;
            len = len_;
        }
        std::string name;
        int         start;
        int         len;
    };
    std::list<Section> _sections;

    // configuration data
    class Conf_element {
    public:
        Conf_element(int from_, int to_)
        {
            from = from_;
            to = to_;
        }
        int from;
        int to;
    };
    std::list<Conf_element> conf_single;
    std::list<Conf_element> conf_mult;
    std::list<Conf_element> conf_single1;
    std::list<Conf_element> conf_mult1;

    bool read_hex(FILE *fp);
    bool read_cfb(FILE *fp);
    bool read_cft(FILE *fp);
    bool read_cff(FILE *fp);
    bool read_cin(FILE *fp);

    bool dump_section(FILE *fp, const char *name, int beg, int len);
    bool set_version(const char *ver);
    void correct_i2c_addr(u_int16_t& i2c_slave);
    void jump_to_next_eeprom(int& current);
    void gather_conf(Mem& m, std::list<Conf_element>& singl,
                     std::list<Conf_element>& mult);
    void add_mult_data(std::list<Conf_element>& mult, int from, int to);
    void fill_ucode(int& current, int chunk);
    void fill_data(Mem& m, std::list<Conf_element>& singl,
                   std::list<Conf_element>& mult, int& current);

    std::string      _fwdir;
    int              ee_size;       // Size of each EEPROM in bytes
    int              ee_amount;     // Amount of EEPROMs
    int              ee_first_addr; // First EEPROM I2C slave address
    Ucode            ucode[10];     // Ucode chunks
    int              ucode_chunks;  // Real amount of ucode chunks
    std::string      version;       // Ucode version
    Board::info      *_info;
    Board::more_info *_more_info;
    std::list<std::string> vli;
    Mem              _mem;
    Mem              _mem1;

    u_int32_t              _guid_offs;
    u_int16_t              _guid_i2c;
    u_int32_t              _bsn_offs;
    u_int16_t              _bsn_i2c;

    std::vector<u_int8_t>  _ee_image;    // EEPROM image
    std::vector<bool>      _ee_use;      // EEPROM usage flags
    void Set(unsigned int addr, u_int8_t value)
    {
        if (addr+1 > _ee_image.size())
        {
            _ee_image.resize(addr+1, 0);
            _ee_use.resize(addr+1, false);
        }
        _ee_image[addr] = value;
        _ee_use[addr] = true;
    }
    void Set2(int addr, u_int16_t value)
    {
        SWAP16(value);
        unsigned char *p = (unsigned char *)&value;
        for(unsigned int i=0; i<sizeof(u_int16_t); i++)
            Set(addr++, *p++);
    }
    void Set4(int addr, u_int32_t value)
    {
        SWAP32(value);
        unsigned char *p = (unsigned char *)&value;
        for(unsigned int i=0; i<sizeof(u_int32_t); i++)
            Set(addr++, *p++);
    }

};

#endif
